#ifndef _DBITARRAY_H
#define _DBITARRAY_H

// This file was automatically generated by objch.

//==============================================================================
//
//          DBitArray - the bit array data type in the ofc-library
//
//               Copyright (C) 2003  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2005/11/12 10:04:03 $ $Revision: 1.8 $
//
//==============================================================================

#include "ofc/config.h"


// The bit array data type implements a number of methods for creating, 
// manipulating and checking bit sets. 

@interface DBitArray : Object
{
@private
  unsigned char *_array;  // the bits for the array
  int            _min;    // the minimum value in the bit array
  int            _max;    // the maximum value in the bit array
  int            _size;   // the size of the bit array
}


//// Constructors

//
// Initialise to an empty bit array with range 0..255
//
// @return the object
//
- (DBitArray *) init;

//
// Initialise to an empty bit array
//
// @param min      the minimum value in the bit array
// @param max      the maximum value in the bit array
//
// @return the object
//
- (DBitArray *) init :(int) min :(int) max;


//// Copy related methods

//
// Deepen a copy of the bit array object
//
// @return the object
//
- deepen;


//// Destructor

//
// Free the bit array
//
// @return the object
//
- free;


//// Member methods

//
// Return the minimum value in the bit array
//
// @return the minumum value
//
- (int) min;

//
// Return the maximum value in the bit array
//
// @return the maximum value
//
- (int) max;


//// Main methods

//
// Set one value in the bit array
//
// @param val       the value to set
//
// @return the object
//
- (DBitArray *) set :(int) val;

//
// Set a range of values in the bit array
//
// @param from     the start of the bit array (incl.)
// @param to       the end of the bit array (incl.)
//
// @return the object
//
- (DBitArray *) set :(int) from :(int) to;

//
// Set a range of values, stepwise in the bit array
//
// @param from     the start of the bit set (incl.)
// @param to       the end of the bit set (incl.)
// @param step     the step for the bit set (> 0)
//
// @return the object
//
- (DBitArray *) set :(int) from :(int) to :(unsigned) step;

//
// Reset the whole bit array
//
// @return the object
//
- (DBitArray *) reset;

//
// Reset one value in the bit array
//
// @param val       the value to reset
//
// @return the object
//
- (DBitArray *) reset :(int) val;

//
// Reset a range of values in the bit array
//
// @param from     the start of the bit set (incl.)
// @param to       the end of the bit set (incl.)
//
// @return the object
//
- (DBitArray *) reset :(int) from :(int) to;

//
// Reset a range of values, stepwise in the bit array
//
// @param from     the start of the bit set (incl.)
// @param to       the end of the bit set (incl.)
// @param step     the step for the bit set (> 0)
//
// @return the object
//
- (DBitArray *) reset :(int) from :(int) to :(unsigned) step;


//// Checking methods

//
// Check if a value is set in the bit array
//
// @param val      the value to be checked
//
// @return is the value set in the bit array ?
//
- (BOOL) has :(int) val;

//
// Count the number of values set
//
// @return the number of values set
//
- (int) count ;

@end
#endif
