#ifndef _DCALENDAR_H
#define _DCALENDAR_H

// This file was automatically generated by objch.

//==============================================================================
//
//           DCalendar - the calendar class in the ofc-library
//
//               Copyright (C) 2003  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2005/11/12 10:04:03 $ $Revision: 1.15 $
//
//==============================================================================

#include "ofc/config.h"

#include "ofc/DTextable.h"
#include "ofc/DText.h"
#include "ofc/DDatable.h"
#include "ofc/DData.h"
#include "ofc/DDateTime.h"


#define DCL_SUNDAY      (DDT_SUNDAY)
#define DCL_MONDAY      (DDT_MONDAY)
#define DCL_TUESDAY     (DDT_TUESDAY)
#define DCL_WEDNESDAY   (DDT_WEDNESDAY)
#define DCL_THURSDAY    (DDT_THURSDAY)
#define DCL_FRIDAY      (DDT_FRIDAY)
#define DCL_SATURDAY    (DDT_SATURDAY)

#define DCL_ALL_MONTHS -1


// The calendar class implements a number of methods for generating calendars.

@interface DCalendar : Object <DTextable,DDatable>
{
@private
  int          _year;    // the year for the calendar
  int          _month;   // the month for the calendar (or DCL_ALL_MONTHS)
}


//// Constructors

//
// Initialise to a default calendar (2003, all months)
//
// @return the object
//
- (DCalendar *) init;

//
// Initialise to an all months calendar for a year
//
// @param year     the year for the calendar
//
// @return the object
//
- (DCalendar *) init :(int) year;

//
// Initialise to a month calendar
//
// @param year     the year for the calendar
// @param month    the month for the calendar
//
// @return the object
//
- (DCalendar *) init :(int) year :(int) month;


//// First week day methods

//
// Get the first week day (initialy DCL_SUNDAY)
//
// @return the first week day (DCL_SUNDAY or DCL_MONDAY)
//
+ (int) firstWeekDay;

//
// Set the first week day
//
// @param day      the day number for the first week day (DCL_SUNDAY or DCL_MONDAY)
//
+ (void) firstWeekDay :(int) day;


//// Year methods

//
// Get the year of the calendar
//
// @return the year
//
- (int) year;

//
// Set the year for the calendar
//
// @param year      the year for the calendar
//
// @return the object
//
- (DCalendar *) year :(int) year;

//
// Check if a year is a leap year
//
// @param year     the year to be checked
//
// @return is it ?
//
+ (BOOL) isLeapYear :(int) year;

//
// Return the number of leap years in a year range
//
// @param from     the start year
// @param to       the last year (incl, to >= from)
//
// @return the number of leap years
//
+ (int) leapYears :(int) from :(int) to;


//// Month methods

//
// Return the month of the calendar
//
// @return the month (or DCL_ALL_MONTHS)
//
- (int) month;

//
// Set the month for the calendar
//
// @param month    the month for the calender (or DCL_ALL_MONTHS)
//
// @return the object
//
- (DCalendar *) month :(int) month;

//
// Return the number of days in a month
//
// @param year     the year of the month
// @param month    the month (1..12)
//
// @return the number of days in the month (1..31) or (-1)
//
+ (int) daysInMonth :(int) year :(int) month;


//// Day methods

//
// Return the week day of a date
//
// @param year     the year  (1970..)
// @param month    the month (1..12)
// @param day      the day   (1..31)
//
// @return the week day (DCL_SUNDAY..DCL_SATURDAY)
//
+ (int) weekDay :(int) year :(int) month :(int) day;


//// Calendar methods

//
// Return the calendar in a data string
//
// @return a (new) data string with the calendar (7 elements per week, 0 = day outside month)
//
- (DData *) toData;

//
// Return the calendar in a text string
//
// @return a (new) text string object
//
- (DText *) toText;

@end
#endif
