#ifndef _DCIRCLE_H
#define _DCIRCLE_H

// This file was automatically generated by objch.

//==============================================================================
//
//        DCircle - the circular buffer collection in the ofc-library
//
//               Copyright (C) 2003  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2005/01/31 06:44:29 $ $Revision: 1.5 $
//
//==============================================================================

#include "ofc/config.h"

#include "ofc/DStackable.h"
#include "ofc/DQueuable.h"

//
// The circular buffer collection stores objects in an circular buffer that can grow. 
// Objects can be read by an index, which can be negative for accessing elements from 
// the end. This collection can be used for implementing a queue (FIFO) or a stack (LIFO).
//

@interface DCircle : Object <DStackable,DQueuable>
{
@private
  id            *_objects;  // the stored objects
  long           _size;     // the size of the buffer
  long           _first;    // the first element in the buffer (or -1 for empty)
  long           _next;     // the next element in the buffer
}

//// Constructors

//
// Initialise to an empty buffer
//
// @return the object
//
- (DCircle *) init;

//
// Initialise to a circular buffer with an initial size
//
// @param size     the size of the buffer
//
// @return the object
//
- (DCircle *) init :(long) size;


//// Copy related methods

//
// Deepen a copy of the buffer
//
// @return the object
//
- deepen;


//// Destructor

//
// Free the buffer, not the stored objects
//
// @return the object
//
- shallowFree;

//
// (Deep) free the buffer (incl. the stored objects)
//
// @return the object
//
- free;


//// Member methods

//
// Return the size of the buffer
//
// @return the size
//
- (long) size;

//
// Set the size for the buffer
//
// @param size       the new, larger size
//
// @return the object
//
- (DCircle *) size :(long) size;

//
// Return the number of stored objects
// 
// @return the length
// 
- (long) length;

//
// Check if the buffer is empty
// 
// @return is it ?
// 
- (BOOL) isEmpty;

//
// Check if the buffer is full
// 
// @return is it ?
// 
- (BOOL) isFull;


//// Index methods

//
// Check if an index is valid
//
// @param index   the index
//
// @return is it valid ?
//
- (BOOL) isValid :(long) index;

//
// Return a reference to the indexth object
//
// @param index      the index
//
// @return the object (or nil if invalid index)
//
- (id) get :(long) index;


//// Stack/LIFO methods

//
// Push an object on the stack (LIFO)
//
// @param obj      the object to be pushed
//
// @return success (or buffer full)
//
- (BOOL) push :(id) obj;

//
// Pop an object from the stack (LIFO)
//
// @return the object (or nil if empty)
//
- (id) pop;

//
// Return the object on top of the stack (LIFO)
// (without changing the top of the stack)
//
// @return the object (or nil if empty)
//
- (id) tos;


//// Queue/FIFO Methods

//
// Put an object in the queue (FIFO)
//
// @param obj      the object 
//
// @return success (or buffer full)
//
- (BOOL) enqueue :(id) obj;

//
// Get an object from the queue (FIFO)
//
// @param obj      the object 
//
// @return the object (or nil for buffer empty)
//
- (id) dequeue ;


//// Selector methods

//
// Perform a selector on each object in the circular buffer, 
// starting from the first item in the buffer
//
// @param sel      the selector
//
// @return the object
//
- (DCircle *) each :(SEL) sel;

//
// Perform a selector on each object in the circular buffer,
// starting from the last item in the buffer
//
// @param sel      the selector
//
// @return the object
//
- (DCircle *) reach :(SEL) sel;

@end
#endif
