#ifndef _DCONFIG_H
#define _DCONFIG_H

// This file was automatically generated by objch.

//==============================================================================
//
//      DConfig - the config parser, writer and storage in the ofc-library
//
//               Copyright (C) 2004  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2005/11/14 19:55:41 $ $Revision: 1.7 $
//
//==============================================================================

#include "ofc/config.h"
#include "ofc/DLexer.h"

#ifdef HAVE_DLEXER
# define HAVE_DCONFIG (1)
#endif


#include "ofc/DTree.h"

#ifdef HAVE_DCONFIG

#define DCH_CLOSE_EXPECTED   (1) // end of section ']' expected
#define DCH_SECTION_EXPECTED (2) // section name expected
#define DCH_ASSIGN_EXPECTED  (3) // : or = expected after option name
#define DCH_CHAR_UNEXPECTED  (4) // unexpected character


//
// The DConfigHandler protocol specifies the methods a config handler
// must implement for processing the contents of a config file.
//

@protocol DConfigHandler

//
// Process the start of the config file
// 
// @return success
// 
- (BOOL) startConfig;

//
// Process the end of the config file
// 
// @return success
//
- (BOOL) endConfig;

//
// Process a (start of a) section
// 
// @param name   the name of the section
// 
// @return success
// 
- (BOOL) section :(const char *) name;

//
// Process an option
// 
// @param section  the name of the section
// @param name     the name of the option
// @param value    the value of the option
// 
// @return success
// 
- (BOOL) option :(const char *) section :(const char *) name :(const char *) value;

//
// Process a comment in the config file
// 
// @param comment  the comment text
// 
// @return success
// 
- (BOOL) comment :(const char *) comment;

//
// An error is found during the parsing of the file
// 
// @param number       the error number
// @param name         the name of the config file
// @param lineNumber   the line number in the config file
// @param columnNumber the column number in the config file
// 
// @return success
// 
- (void) error :(int) number :(const char *) name :(int) lineNumber :(int) columnNumber;

@end

//
// The DConfigReader class implements methods for parsing a config file.
//

@interface DConfigReader : Object
{
@private
  DLexer              *_lexer;   // the lexer used to parse the file
}
#endif

#ifdef HAVE_DCONFIG

//// Constructors

//
// Initialise a config reader
//
// @return the object
//
- (DConfigReader *) init;


//// Copy constructor

//
// Deepen a copy of the object (not implemented)
//
// @return the object
//
- deepen;


//// Deconstructor

//
// Free the object
//
// @return the object
//
- free;


//// Parser methods

//
// Parse a config file
// 
// @param source   the source of the config file
// @param name     the name of the config file
// @param handler  the config handler
// 
// @return success
// 
- (BOOL) parse :(id <DTextReadable>) source :(const char *) name :(id <DConfigHandler>) handler;

@end
#endif

#ifdef HAVE_DCONFIG
//
// The config writer class implements methods for writing a config file.
//
   
@interface DConfigWriter : Object <DConfigHandler>
{
@private
  id <DTextWritable>  _dest;     // the config file
  DText              *_section;  // the current section
}
#endif
#ifdef HAVE_DCONFIG

//// Constructors

//
// Initialise a config writer
//
// @return the object
//
- (DConfigWriter *) init;

//
// Initialise a config writer with a config file
// 
// @param destination  the config file
//
// @return the object
//
- (DConfigWriter *) init :(id <DTextWritable>) destination;


//// Deconstructor

//
// Free the object
//
// @return the object
//
- free;


//// Writer methods

//
// Start the writing of a config file
// 
// @param destination  the config file
// 
// @return the object
// 
- (BOOL) startConfig :(id <DTextWritable>) destination;

//
// Process the start of the config file
// 
// @return success
// 
- (BOOL) startConfig;

//
// Process the end of the config file
// 
// @return success
//
- (BOOL) endConfig;

//
// Write the (start of a) section
// 
// @param name   the name of the section
// 
// @return success
// 
- (BOOL) section :(const char *) name;

//
// Write an option
// 
// @param section  the name of the section
// @param name     the name of the option
// @param value    the value of the option
// 
// @return success
// 
- (BOOL) option :(const char *) section :(const char *) name :(const char *) value;

//
// Write a comment in the config file
// 
// @param comment  the comment text
// 
// @return success
// 
- (BOOL) comment :(const char *) comment;

#ifdef WIN32
#endif
//
// Report an error during the writing of the file
// 
// @param number       the error number
// @param name         the name of the config file
// @param lineNumber   the line number in the config file
// @param columnNumber the column number in the config file
// 
// @return success
// 
- (void) error :(int) number :(const char *) name :(int) lineNumber :(int) columnNumber;;

@end
#endif

#ifdef HAVE_DCONFIG
// The Config class implements methods for storing config options,
// including reading and writing to a config file, and changing and
// removing options. Config options are structured in sections with 
// multiple options with values.
   
@interface DConfigTree : Object <DConfigHandler>
{
@private
  DTree         *_config; // the options in a tree of sections
  DTreeIterator *_iter;   // the iterator in the config tree
}
#endif
#ifdef HAVE_DCONFIG

//// Constructors

//
// Initialise a config tree
//
// @return the object
//
- (DConfigTree *) init;

//
// Initialise a config tree with a config source
// 
// @param source     the config source
// @param filename   the name of the config source
//
// @return the object
//
- (DConfigTree *) init :(id <DTextReadable>) source :(const char *) name;


//// Copy constructor

//
// Deepen a copy of the object (not implemented)
//
// @return the object
//
- deepen;


//// Deconstructor

//
// Free the object
//
// @return the object
//
- free;


//// Config tree reading and writing

//
// Read the config from a source
//  
// @param source the source of the config
// @param name   the name of the source
// 
// @return success
// 
- (BOOL) read :(id <DTextReadable>) source :(const char *) name;

//
// Write the config to a destination
// 
// @param destination the destination of the config
// @param name        the name of the destination (not used)
// 
// @return success
// 
- (BOOL) write :(id <DTextWritable>) destination :(const char *) name;


//// Options methods

//
// Set the value for a section,option
// 
// @param section   the section for the option
// @param option    the option
// @param value     the value for the option
// 
// @return success
// 
- (BOOL) set :(const char *) section :(const char *) option :(const char *) value;

//
// Get the value of a section,option
// 
// @param section   the section for the option
// @param option    the option
// 
// @return the value (or NULL)
// 
- (const char *) get :(const char *) section :(const char *) option;

//
// Check if the config tree has a section
// 
// @param section   the section
// 
// @return has it ?
// 
- (BOOL) has :(const char *) section;

//
// Check if the config tree has a section,option
// 
// @param section   the section
// @param option    the option
// 
// @return has it ?
// 
- (BOOL) has :(const char *) section :(const char *) option;

//
// Remove a section (and all its options) from the config tree
// 
// @param section   the section
// 
// @return success
// 
- (BOOL) remove :(const char *) section;

//
// Remove a section,option from the config tree
// 
// @param section   the section
// @param option    the option
// 
// @return success
// 
- (BOOL) remove :(const char *) section :(const char *) option;


//// List methods

//
// Return a list with all the sections in the config tree
// 
// @return a (new) list with all the sections
// 
- (DList *) sections;

//
// Return a list with all the options in a section
// 
// @param section   the section for the opstions
// 
// @return a (new) list with all the options (or nil for unkown section)
// 
- (DList *) options :(const char *) section;


//// Call back methods for the config parser (private)

//
// Process the start of the config file
// 
// @return success
// 
- (BOOL) startConfig;

//
// Process the end of the config file
// 
// @return success
//
- (BOOL) endConfig;

//
// Process the (start of a) section
// 
// @param name   the name of the section
// 
// @return success
// 
- (BOOL) section :(const char *) name;

//
// Process an option
// 
// @param section  the name of the section
// @param option   the name of the option
// @param value    the value of the option
// 
// @return success
// 
- (BOOL) option :(const char *) section :(const char *) option :(const char *) value;

//
// Write a comment in the config file
// 
// @param comment  the comment text
// 
// @return success
// 
- (BOOL) comment :(const char *) comment;

//
// An error is found during the parsing of the file
// 
// @param number       the error number
// @param name         the name of the config file
// @param lineNumber   the line number in the config file
// @param columnNumber the column number in the config file
// 
// @return success
// 
- (void) error :(int) number :(const char *) name :(int) lineNumber :(int) columnNumber;;

@end
#endif
#endif
