#ifndef _DCUBE_H
#define _DCUBE_H

// This file was automatically generated by objch.

//==============================================================================
//
//         DCube - the 3d - array collection in the ofc-library
//
//               Copyright (C) 2003  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2005/01/31 06:44:29 $ $Revision: 1.3 $
//
//==============================================================================

#include "ofc/config.h"


// The cube collection stores objects in a 3d array that can grow. 
// Objects can be accessed by an index. Due to the easy access there 
// is no iterator.

@interface DCube : Object
{
@private
  id            *_objects;  // the stored objects
  int            _columns;  // the number of columns
  int            _rows;     // the number of rows
  int            _layers;   // the number of layers
  long           _length;   // the length of the cube
}


//// Constructors

//
// Initialise an empty cube
//
// @return the object
//
- (DCube *) init;

//
// Initialise a cube with columns, rows and layers
//
// @param columns  the number of columns
// @param rows     the number of rows
// @param layers   the number of layers
//
// @return the object
//
- (DCube *) init :(int) columns :(int) rows :(int) layers;


//// Copy related methods

//
// Deepen a copy of the cube
//
// @return the object
//
- deepen;


//// Destructor

//
// Free the cube, not the stored objects
//
// @return the object
//
- shallowFree;

//
// (Deep) free the cube (incl. the stored objects)
//
// @return the object
//
- free;


//// Member methods

//
// Return the length of the cube (columns * rows * layers)
//
// @return the length
//
- (long) length;

//
// Return the number of columns
// 
// @return the number of columns
// 
- (int) columns;

//
// Set the number of columns (only growth)
// 
// @param columns  the number of columns
// 
// @return the object
// 
- (DCube *) columns :(int) columns;

//
// Return the number of rows
// 
// @return the number of rows
// 
- (int) rows;

//
// Set the number of rows (only growth)
// 
// @param rows     the number of rows
// 
// @return the object
// 
- (DCube *) rows :(int) rows;

//
// Return the number of layers
// 
// @return the number of layers
// 
- (int) layers;

//
// Set the number of layers (only growth)
// 
// @param layers   the number of layers
// 
// @return the object
// 
- (DCube *) layers :(int) layers;

//
// Check if an index is valid
//
// @param col      the column (0..)
// @param row      the row (0..)
// @param layer    the layer (0..)
//
// @return is it valid ?
//
- (BOOL) isValid :(int) col :(int) row :(int) layer;


//// Main methods

//
// Set an object in the cube
//
// @param col      the column (0..)
// @param row      the row (0..)
// @param layer    the layer (0..)
// @param obj      the object to be set
//
// @return the previous stored object in the cube (or nil)
//
- (id) set :(int) col :(int) row :(int) layer :(id) obj;

//
// Get an object from the cube
//
// @param col      the column (0..)
// @param row      the row (0..)
// @param layer    the layer (0..)
//
// @return the object in the cube (or nil)
//
- (id) get :(int) col :(int) row :(int) layer;

//
// Count the number of occurences of an object
//
// @param obj      the object to be counted
//
// @return the number of occurences
//
- (long) count :(id) obj;

//
// Check if an object is in the cube
//
// @param obj      the object
//
// @return is it in the cube ?
//
- (BOOL) has :(id) obj;

//
// Perform a selector on each object in the cube
//
// @param sel      the selector
//
// @return the object
//
- (DCube *) each :(SEL) sel;

@end
#endif
