#ifndef _DDIRECTORY_H
#define _DDIRECTORY_H

// This file was automatically generated by objch.

//==============================================================================
//
//            DDirectory - the directory class in the ofc-library
//
//               Copyright (C) 2004  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2005/12/27 08:28:11 $ $Revision: 1.13 $
//
//==============================================================================
#ifdef WIN32
#endif

#include "ofc/config.h"

#include "ofc/DText.h"
#include "ofc/DList.h"
#include "ofc/DTextFilterable.h"

//
// The DDirectory class implements a number of methods for using directory paths.
// The paths are not checked by the ofc-library; this is left to the underlying os-specific
// functions. This class has also a number of class methods for creating, deleting, renaming 
// and scanning directories.
//

@interface DDirectory : Object
{
@private
  DText          *_path;    // the path (and name) of the directory
}


//// Constructors

//
// Initialise an empty directory object
//
// @return the object
//
- (DDirectory *) init;

//
// Initialise the object with a directory path
//
// @return the object
//
- (DDirectory *) init :(const char *) path;


//// Deconstructor

//
// Free the object
//
// @return the file object
//
- free;


//// Copy related methods

//
// Deepen a copied object 
//
// @return the object
//
- deepen;


//// Class methods

//
// Check if the character is a drive separator
// (there is no check for misuse of ':' on WIN32)
// 
// @param ch           the character
// 
// @return is it a drive separator ?
// 
+ (BOOL) isDriveSeparator :(char) ch;

#ifdef WIN32
#endif
//
// Check if the character is a path separator 
// 
// @param ch           the character 
// 
// @return is it a path separator ?
// 
+ (BOOL) isPathSeparator :(char) ch;

#ifdef WIN32
#endif
//
// Create a directory
// 
// @param path         the path
// 
// @return success
// 
+ (BOOL) create :(const char *) path;

#ifdef WIN32
#endif
//
// Move/Rename a directory
// 
// @param path         the current path
// @param newPath      the new path
// 
// @return success
// 
+ (BOOL) move :(const char *) path :(const char *) newPath;

#ifdef WIN32
#endif
// 
// Remove a directory
// 
// @param path         the path 
// 
// @return success
// 
+ (BOOL) remove :(const char *) path;

#ifdef WIN32
#endif
//
// Return the current working directory
// 
// @return a (new) directory object with the current directory (or nil)
// 
+ (DDirectory *) current;

#ifdef WIN32
#endif
//
// Set the current working directory
// 
// @param path         the path for the current directory
// 
// @return success
//
+ (BOOL) current :(const char *) path;

#ifdef WIN32
#endif
//
// Return the temp directory
// 
// @return a (new) DDirectory object with the temp directory path (or nil)
// 
+ (DDirectory *) temp;

#ifdef WIN32
#endif
//
// Check if the directory exist
// 
// @param path         the directory path
// 
// @return does it ?
// 
+ (BOOL) exist :(const char *) path;

#ifdef WIN32
#endif
//
// Return a list with the filenames in the directory
// 
// @param path         the directory path
// @param filter       the filter for the names (or nil for no filter)
// 
// @return a (new) list met DText objects (or nil)
// 
+ (DList *) childs :(const char *) path :(id <DTextFilterable>) filter;

#ifdef WIN32
#endif
//
// Return the last error for the class methods
// 
// @return the last error
// 
+ (int) error;


//// Member methods

//
// Check if the directory has an absolute path
// 
// @return is the path absolute ?
// 
- (BOOL) isAbsolute;

#ifdef WIN32
#endif
//
// Check if the directory path is empty
// 
// @return is it ?
// 
- (BOOL) isEmpty;

//
// Return the drive letter
// 
// @return the drive letter (or EOS if not present)
// 
- (char) drive;

#ifdef WIN32
#endif
//
// Set the path for the directory
// 
// @param path      the path for the directory
// 
// @return the object
//
- (DDirectory *) path :(const char *) path;

//
// Return the path (and name) of the directory
//
// @return the path
//
- (const char *) path;

//
// Return the directory name in the path
// 
// @return (a reference to) the name (or NULL)
//
- (const char *) name;

//
// Add a subdirectory to the directory object
// 
// @param name      the name of the subdirectory
// 
// @return the object
// 
- (DDirectory *) child :(const char *) name;

#ifdef WIN32
#endif
//
// Move the directory path to the parent directory
// 
// @return was there a parent directory ?
// 
- (BOOL) parent;

//
// Split up the path in a list of directory names
// 
// @return a (new) list with the the names (in DText objects)
// 
- (DList *) names;

#ifdef WIN32
#endif
//
// Build the path with a list of directory names
// 
// @param names        a list of objects that respond to the textable protocol
// 
// @return the object
// 
- (DDirectory *) names :(DList *) names;

@end
#endif
