#ifndef _DDOUBLEARRAY_H
#define _DDOUBLEARRAY_H

// This file was automatically generated by objch.

//==============================================================================
//
//  DDoubleArray - the one dimensional array of doubles class in the ofc-library
//
//               Copyright (C) 2005  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2005/06/21 18:29:19 $ $Revision: 1.1 $
//
//==============================================================================

#include "ofc/config.h"

#include "ofc/DTextable.h"
#include "ofc/DComparable.h"
#include "ofc/DSizeable.h"
#include "ofc/DParsable.h"

#include "ofc/DText.h"

//
// The DDoubleArray class implements an one dimensional array of double. It is of 
// course possible to create an array of double with the classes DArray and DDouble, 
// but this is not very efficient and a little bit clumsy. This class also implements some
// statistical methods.
// 

@interface DDoubleArray : Object <DTextable,DComparable,DSizeable,DParsable>
{
@private
  unsigned long  _size;     // the maximum length of the double array
  unsigned       _extra;    // the extra size amount
  unsigned long  _length;   // the current length of the double array
  double        *_array;    // the array of doubles
}


//// Constructors

//
// Initialise to an empty array
//
// @return the object
//
- (DDoubleArray *) init;

//
// Initialise the object with an array of doubles
//
// @param doubles  an array with the doubles
// @param length   the length of the array
//
// @return the object
//
- (DDoubleArray *) init :(const double *) doubles :(unsigned long) length;


//// Copy related methods

//
// Deepen a copy of the object
//
// @return the object
//
- deepen;


//// Deconstructor

//
// Free the object
//
// @return the object
//
- free;


//// Member methods

//
// Check if the array is empty
//
// @return Is it empty ?
//
- (BOOL) isEmpty;

//
// Return the length of the array
//
// @return the length
//
- (unsigned long) length;

//
// Return the pointer to the array of doubles
//
// @return the pointer to the array
//
- (const double *) array;


//// Size methods

//
// Insure the size for the array
//
// @param size     the requested size (in elements) for the array
//
// @return the object
//
- (DDoubleArray *) size :(unsigned long) size;

//
// Set the amount of extra size increase if the size must be extended
//
// @param extra    the amount (def. 0)
//
// @return the object
//
- (DDoubleArray *) extra :(unsigned) extra;


//// DTextable protocol methods

//
// Convert the object to a text string 
// 
// @return a (new) text string object
//
- (DText *) toText;


//// DParsable protocol methods

//
// Parse a string for an array of doubles: <double>,<double>;<double>
// 
// @param cstr     the string to be parsed (moved to the first non-parsable character)
// 
// @return the result (0, ERANGE, ENODATA)
//
- (int) fromString :(char **) cstr;


//// Array setters

//
// Clear the array
// 
// @return the object
// 
- (DDoubleArray *) clear;

//
// Set an array of doubles in the object
//
// @param doubles  the array with the doubles
// @param length   the length of the array
//
// @return the object
//
- (DDoubleArray *) set :(const double *) doubles :(unsigned long) length;


//// Element methods

//
// Put a double at the given index
//
// @param index     the index (-length..+length)
// @param value     the double
//
// @return the object
//
- (DDoubleArray *) put :(long) index :(double) value  ;

//
// Get the double at the given index
//
// @param index    the index 
//
// @return the double from the array
//
- (double) get :(long) index;

// 
// Insert a double at the given index
// 
// @param index    the index
// @param value    the double
// 
// @return the object
//
- (DDoubleArray *) insert :(long) index :(double) value;

//
// Delete the double at the given index
//
// @param index    the index
//
// @return the object
//
- (DDoubleArray *) delete :(long) index;


//// Slice methods

//
// Insert an array of doubles in a part of the array in the object
//
// @param from     the start index
// @param to       the end index (to >= (from-1))
// @param doubles  the array with the doubles
// @param length   the length of the array
//
// @return the object
//
- (DDoubleArray *) insert :(long) from :(long) to :(const double *) doubles :(unsigned long) length;

//
// Return a sub array
//
// @param from     the start index
// @param to       the end index (to >= from)
//
// @return a (new) double array object
//
- (DDoubleArray *) get :(long) from :(long) to;

//
// Delete a range in the array
//
// @param from     the start index
// @param to       the end index (to >= from)
//
// @return the object
//
- (DDoubleArray *) delete :(long) from :(long) to;


//// Array manipulation methods

//
// Append an array to the object
//
// @param doubles  the array with the doubles
// @param length   the length of the array
//
// @return the object
//
- (DDoubleArray *) append :(const double *) doubles :(unsigned long) length;

//
// Prepend an array in front of the object
//
// @param doubles  the array with the doubles
// @param length   the length of the array
//
// @return the object
//
- (DDoubleArray *) prepend :(const double *) doubles :(unsigned long) length;


//// Stack methods

//
// Push one double at the end of the array
// 
// @param value    the double
//
// @return the object
//
- (DDoubleArray *) push :(double) value;

//
// Pop one double from the end of the array
//
// @return the double that is pulled (or 0.0 for empty stack)
//
- (double) pop;

//
// Return the double at the end of the array
// 
// @return the double (or 0.0 for empty stack)
// 
- (double) tos;


//// Queue methods (FIFO)

//
// Put a double at the start of the array 
//
// @param value    the double 
// 
// @return success
// 
- (BOOL) enqueue :(double) value;

//
// Get the double at the end of the array
// 
// @return the double (or 0.0 for empty queue)
// 
- (double) dequeue;


//// Comparable protocol methods

//
// Compare the double array object with another double array object
//
// @param other    the other object (!= nil)
//
// @return the compare result (-1, 0, 1)
//
- (int) compare :(DDoubleArray *) other;


//// Comparison methods

//
// Compare the object with an array of doubles
//
// @param doubles  the array with the doubles
// @param length   the length of the array
//
// @return the compare result (-1, 0, 1)
//
- (int) bcompare :(const double *) doubles :(unsigned long) length;


//// Statistical methods

//
// Count the number of occurrences of a double in the object
//
// @param search   the count value
// @param from     the start index 
// @param to       the end index (to >= from)
//
// @return the number of occurrences
//
- (unsigned long) count :(double) search :(long) from :(long) to  ;

//
// Return the first index where the search double is found
//
// @param search   the search value
// @param from     the start index
// @param to       the end index (to >= from)
//
// @return the first index or -1 = not found
//
- (long) index :(double) search :(long) from :(long) to ;

//
// Return the last index where the search double is found
//
// @param search   the search value
// @param from     the start index
// @param to       the end index (to >= from)
//
// @return the last index or -1 = not found
//
- (long) rindex :(double) search :(long) from :(long) to;

//
// Calculate the sum in a range of the double array
// 
// @param from     the start index
// @param to       the end index (to >= from)
// 
// @return the sum of the doubles
// 
- (double) sum :(long) from :(long) to;

//
// Determine the maximum double in a range of the double array
//
// @param from     the start index
// @param to       the end index (to >= from)
// 
// @return the maximum double
// 
- (double) max :(long) from :(long) to;

//
// Determine the minimum double in a range of the double array
//
// @param from     the start index
// @param to       the end index (to >= from)
// 
// @return the minimum double
// 
- (double) min :(long) from :(long) to;

//
// Calculate the average in a range of the double array
// 
// @param from     the start index
// @param to       the end index (to >= from)
//
// @return the average (or 0.0)
// 
- (double) average :(long) from :(long) to;

// 
// Calculate the variance in a range of the double array
// 
// @param from     the start index
// @param to       the end index (to >= from)
// 
// @return the variance
// 
- (double) variance :(long) from :(long) to;

//
// Calculate the standard deviation in a range of the double array
// 
// @param from     the start index
// @param to       the end index (to >= from)
// 
// @return the standard deviation
// 
- (double) standardDeviation :(long) from :(long) to;

//
// Sort a range in the double array
//
// @param from     the start index
// @param to       the end index (to > from)
// 
// @return the object
// 
- (DDoubleArray *) sort :(long) from :(long) to;

//
// Invert (mirror) a range in the double array
// 
// @param from     the start index
// @param to       the end index (to >= from)
// 
// @return the object
// 
- (DDoubleArray *) invert :(long) from :(long) to;

@end
#endif
