#ifndef _DFIXEDPOINT_H
#define _DFIXEDPOINT_H

// This file was automatically generated by objch.

//==============================================================================
//
//       DFixedPoint - the fixed point number class in the ofc-library
//
//               Copyright (C) 2005  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2005/06/12 05:09:06 $ $Revision: 1.5 $
//
//==============================================================================
// ToDo:
// 1. Conversion to text without the use of a double
// 2. More functions

#include "ofc/config.h"

#include <limits.h>

#include "ofc/DTextable.h"
#include "ofc/DSimpleArithmetic.h"
#include "ofc/DComparable.h"

#include "ofc/DText.h"


#define DFP_MAX_POINT      ((sizeof(long) * CHAR_BIT) - 2)  // maximum value for point

//
// The fixed point class implements a 'floating point number' in a long with a 
// fixed point. The format is ..2^3+2^2+2^1+2^0+2^(-1)+2^(-2)+2^(-3)... For
// example if the point is set to 3 bits, the binary number 11011 equals to
// 3.375.
//

@interface DFixedPoint : Object <DTextable,DSimpleArithmetic,DComparable>
{
@private
  long         _value; // the value of the fixed point
  unsigned     _point; // the number of bits for the point
}


//// Constructors

//
// Initialise to fixed point to zero with no fixed point
//
// @return the object
//
- (DFixedPoint *) init;

//
// Initialise to a fixed point
//
// @param point    the number of bits in the point (0..DFP_MAX_POINT)
//
// @return the object
//
- (DFixedPoint *) init :(unsigned) point;

//
// Initialise to a fixed point
//
// @param value    the value for the fixed point
// @param point    the number of bits in the point
//
// @return the object
//
- (DFixedPoint *) init :(long) value :(unsigned) point;


//// Member methods

//
// Get the point
// 
// @return the number of bits in the point
// 
- (unsigned) point;

//
// Change the precision of the current fixed point by changing the point
//
// @param point      the number of bits in the point (0..DFN_MAX_POINT)
//
// @return the object
//
- (DFixedPoint *) point :(unsigned) point;

//
// Set the fixed point number (using the current point)
//
// @param value    the fixed point number
// 
// @return the object
//
- (DFixedPoint *) set :(long) value;

//
// Set the fixed point number
// 
// @param value  the fixed point number
// @param point  the number of bits for the point
// 
// @return the object
// 
- (DFixedPoint *) set :(long) value :(unsigned) point;

//
// Get the fixed point number
// 
// @return the fixed point number
// 
- (long) get;

//
// Move an other object in this object
//
// @param other    the other object
//
// @return this object
//
- (DFixedPoint *) move :(DFixedPoint *) other;

//
// Normalize the fixed point number by removing trailing zero bits
//
// @return the object
//
- (DFixedPoint *) norm;


//// Methods on fixed point numbers

//
// Add with an other fixed point number 
//
// @param other    the fixed point number
//
// @return the object
//
- (DFixedPoint *) add :(DFixedPoint *) other;

//
// Add two fixed point numbers and store the result in the object
//
// @param src1     the first fixed point number
// @param src2     the second fixed point number
//
// @return the object
//
- (DFixedPoint *) add :(DFixedPoint *) src1 :(DFixedPoint *) src2;

//
// Subract with another fixed point number
//
// @param other     the fixed point number
//
// @return the object
//
- (DFixedPoint *) sub :(DFixedPoint *) other;

//
// Subract two fixed point numbers and store the result in the object
//
// @param src1      the first fixed point number
// @param src2      the second fixed point number
//
// @return the object
//
- (DFixedPoint *) sub :(DFixedPoint *) src1 :(DFixedPoint *) src2;

//
// Multiply with an other fixed point number
//
// @param other    the fixed point number
//
// @return the object
//
- (DFixedPoint *) mul :(DFixedPoint *) other;

//
// Multiply two fixed point numbers and store the result in the object
//
// @param src1     the first fixed point number
// @param src2     the second fixed point number
//
// @return the object
//
- (DFixedPoint *) mul :(DFixedPoint *) src1 :(DFixedPoint *) src2;

//
// Divide with a fixed point number (Note: if the result of the division is zero
// or has to little precision, use point: to scale up the fixed point number)
//
// @param other    the divider
//
// @return the object
//
- (DFixedPoint *) div :(DFixedPoint *) other;

//
// Divide two fixed point numbers and store the result in the object. (Note: 
// if the result of the division is zero or has to little precision, use point:
// to scale up the fixed point number)
//
// @param src1     the fixed point number
// @param src2     the divider
//
// @return the object
//
- (DFixedPoint *) div :(DFixedPoint *) src1 :(DFixedPoint *) src2;


//// Textable protocol methods

//
// Convert to a text string (decimal representation)
//
// @return a (new) text string with the fixed point number
//
- (DText *) toText;


//// Comparable protocol methods

//
// Compare two fixed point numbers
//
// @param other    the other fixed point number (!= nil)
// 
// @return the comparison result (-1 = smaller, 0 = equal, 1 = bigger)
//
- (int) compare :(DFixedPoint *) other;


//// Conversion methods

//
// Convert the fixed point number to a double
// 
// @return the double
// 
- (double) toDouble;

@end
#endif
