#ifndef _DGZIPFILE_H
#define _DGZIPFILE_H

// This file was automatically generated by objch.

//==============================================================================
//
//           DGZipFile - the gzip file class in the ofc-library
//
//               Copyright (C) 2005  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2005/11/14 19:55:41 $ $Revision: 1.4 $
//
//==============================================================================
#ifdef HAVE_DGZIPFILE
#endif

#include "ofc/config.h"

#include "ofc/DText.h"
#include "ofc/DData.h"
#include "ofc/DList.h"
#include "ofc/DTextReadable.h"
#include "ofc/DTextWritable.h"
#include "ofc/DDataReadable.h"
#include "ofc/DDataWritable.h"

#if defined(HAVE_LIBZ) && defined(HAVE_ZLIB_H)
#define HAVE_DGZIPFILE
#endif

#ifdef HAVE_DGZIPFILE

// the seek origins
#define DGZ_SEEK_SET  (0)    // Seek from the start of the file
#define DGZ_SEEK_CUR  (1)    // Seek from the current position in the file

// the strategies
#define DGZ_DEFAULT   (0)    // Default stategy
#define DGZ_FILTERED  (1)    // Filtered data strategy
#define DGZ_HUFFMAN   (2)    // Only huffman compression strategy


// The DGZipFile class implements a number of methods for opening of, writing to, 
// reading from and closing of gzip files.

@interface DGZipFile : Object <DTextReadable,DTextWritable,DDataReadable,DDataWritable>
{
@private
  void           *_file;    // the file pointer
}

#endif
#ifdef HAVE_DGZIPFILE

//// Constructors

//
// Initialise an empty file object
//
// @return the object
//
- (DGZipFile *) init;

//
// Open a gzip file with default level and strategy
//
// @param name     the filename (with extension .gz)
// @param mode     the file mode (r,w)
//
// @return the object
//
- (DGZipFile *) init :(const char *) name :(const char *) mode;

//
// Open a gzip file
//
// @param name     the filename (with extension .gz)
// @param mode     the file mode (r,w)
// @param level    the level of compression
// @param strategy the strategy of compression
//
// @return the object
//
- (DGZipFile *) init :(const char *) name :(const char *) mode :(int) level :(int) strategy;


//// Deconstructor

//
// Free the object
//
// @return the file object
//
- free;


//// Copy related methods

//
// Deepen a copied object (not implemented)
//
// @return the object
//
- deepen;


//// File info methods

//
// Check if the file is open
//
// @return is the file open ?
//
- (BOOL) isOpen;

//
// Return the last error
//
// @return the last error
//
- (int) error;


//// File opening methods

//
// Open a gzip file with default level and strategy
//
// @param name     the filename (with extension .gz)
// @param mode     the file mode (r,w)
//
// @return is the file open ?
//
- (BOOL) open :(const char *) name :(const char *) mode;

//
// Open a gzip file with a level and strategy (note: only output files)
//
// @param name     the filename (with extension .gz)
// @param mode     the file mode (w)
// @param level    the level of compression (0 = none..9 = max,slow)
// @param strategy the strategy of compression (see DGZ..)
//
// @return is the file open ?
//
- (BOOL) open :(const char *) name :(const char *) mode :(int) level :(int) strategy;


//// TextReadable protocol implementation

//
// Read a text string (all available text)
//
// @return a (new) text string
//
- (DText *) readText;

//
// Read a text string 
//
// @param length   the length of the text to be read
//
// @return a (new) text string
//
- (DText *) readText :(long) length;

//
// Read a line of text (till Eof or \n)
//
// @return a (new) text string (or nil for no data)
//
- (DText *) readLine;;

//
// Read a character
//
// @return the character read (EOS for EOF)
//
- (char) readChar;;

//
// Tell the current position in the readable
//
// @return the current position (or -1 for error)
//
- (unsigned long) tell;;

//
// Move the current position in the readable
//
// @param offset   the offset from the origin (in bytes)
// @param origin   the origin for the offset (see DGZ_SEEK..)
//
// @return success
//
- (BOOL) seek :(unsigned long) offset :(int) origin;;

//
// Skip a number of positions
//
// @param offset   the number of bytes to skip
//
// @return success
//
- (BOOL) skip :(unsigned long) offset;;

//
// Check if the end of the gzip file is reached
//
// @return is it?
//
- (BOOL) isEof;;


//// Implementation of DTextWritable

//
// Write text string 
//
// @param text     the text to be written
//
// @return success
//
- (BOOL) writeText :(const char *) text;;

//
// Write line (appending a '\n')
//
// @param text     the text to be written
//
// @return success
//
- (BOOL) writeLine :(const char *) text;

//
// Write a character
//
// @param ch       the character to be written
//
// @return success
//
- (BOOL) writeChar :(char) ch;;


//// Methods for DDataReadable protocol

//
// Read a data string 
//
// @param length   the length of the data
//
// @return the object
//
- (DData *) readData :(unsigned long) length;;

//
// Read a byte
//
// @return the byte read (0 for error)
//
- (unsigned char) readByte;;

//
// Read a short
//
// @return the short
//
- (short) readShort;;

//
// Read a long
//
// @return the long
//
- (long) readLong;;

//
// Read a double
//
// @return the double
//
- (double) readDouble;;


//// DDataWritable protocol implementation

//
// Write a data string 
//
// @param data     the data to be written
// @param length   the length of the data
//
// @return success
//
- (BOOL) writeData :(const unsigned char *) text :(unsigned long) length;;

//
// Write a byte
//
// @param byte     the byte to be written
//
// @return success
//
- (BOOL) writeByte :(unsigned char) byte;;

//
// Write a short 
//
// @param nr       the short to be written
//
// @return success
//
- (BOOL) writeShort :(short) nr;;

//
// Write a long
//
// @param nr       the long to be written
//
// @return success
//
- (BOOL) writeLong :(long) nr;

//
// Write a double
//
// @param nr       the double to be written
//
// @return success
//
- (BOOL) writeDouble :(double) nr;


//// List related methods

//
// Read all lines from a file into a list of strings
//
// @return a (new) list of (new) DText objects
//
- (DList *) readLines;

//
// Writes all the elements ('lines') of text from a list to file
//
// @param list     the list with objects that responds to textable protocol
//
// @return success
//
- (BOOL) writeLines :(DList *) list;


//// File manipulation

//
// Flush the output buffers of the file
//
// @return success
//
- (BOOL) flush;


//// File closing methods

//
// Close the file
//
// @return the file object
//
- (DGZipFile *) close;

@end
#endif
#endif
