#ifndef _DGRAPH_H
#define _DGRAPH_H

// This file was automatically generated by objch.

//==============================================================================
//
//            DGraph - the graph collection in the ofc-library
//
//               Copyright (C) 2005  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2005/07/10 09:02:12 $ $Revision: 1.1 $
//
//==============================================================================

#include "ofc/config.h"

#include "ofc/DList.h"

@class DGraphEdge;  // forward definition

//
// The graph node object stores an object in a node in a graph.
//

@interface DGraphNode : Object
{
@private
  DList         *_ingoing;    // the ingoing edges
  DList         *_outgoing;   // the outgoing edges
  DText         *_name;       // the name (set by DGraph)
  DText         *_label;      // the (graphviz) label
  DText         *_attributes; // the (graphviz) attributes
  id             _object;     // the stored object
  DGraphNode    *_prev;       // the previous node (during shortest path)
  double         _sum;        // the weight sum (during shortest path)
}



//// Constructors

//
// Initialise an empty graph node
//
// @return the object
//
- (DGraphNode *) init;

//
// Initialise a graph node with a label and object
//
// @param label    the label for the node
// @param object   the object
//
// @return the object
//
- (DGraphNode *) init :(const char *) label :(id) object;

//
// Initialise a graph node with a label, attributes and object
//
// @param label        the label for the node
// @param attributes   the (graphviz) attributes for the node
// @param object       the object
//
// @return the object
//
- (DGraphNode *) init :(const char *) label :(const char *) attributes :(id) object;


//// Copy related methods

//
// Deepen a copy of the graph node (not implemented)
//
// @return the object
//
- deepen;


//// Destructor

//
// Free the graph node (the stored edges and object are NOT freed)
//
// @return the object
//
- shallowFree;

//
// Free the graph node (the stored object is freed, the stored 
// edges are NOT freed)
//
// @return the object
//
- free;


//// Member methods

//
// Return the name of the graph node (used by DGraph)
// 
// @return the name of the node
// 
- (const char *) name;

//
// Set the name of the graph node (used by DGraph)
// 
// @param name    the name
// 
// @return the node object
// 
- (DGraphNode *) name :(const char *) name;

//
// Return the label of the graph node
//
// @return the label (or NULL)
//
- (const char *) label;

//
// Set the label of the graph node
//
// @param label   the label (or NULL)
// 
// Return the node object
// 
- (DGraphNode *) label :(const char *) label;

//
// Return the (graphviz) attributes of the node
// 
// @return the attributes (or NULL)
// 
- (const char *) attributes ;

//
// Set the (graphviz) attributes for the node
// 
// @param attributes    the attributes (or NULL)
// 
// @return the node object
// 
- (DGraphNode *) attributes :(const char *) attributes;

//
// Return the stored object in the node
// 
// @return the (stored) object (or nil)
// 
- (id) object;

//
// Set the object for the node
// 
// @param object    the object (or nil)
// 
// @return the previous stored object
// 
- (id) object :(id) object;


//// Path methods (used during shortest path determination)

//
// Reset the path
// 
// @return the node object
// 
- (DGraphNode *) path;

//
// Set the path 
// 
// @param prev    the previous node in the path
// @param sum     the sum of weights in the path
// 
// @return the node object
// 
- (DGraphNode *) path :(DGraphNode *) prev :(double) sum;

//
// Return the previous node in the path
// 
// @return the previous node (or nil)
// 
- (DGraphNode *) prev;

//
// Return the sum of weights in the path
// 
// @return the sum
// 
- (double) sum;


//// Edge methods

//
// Return the ingoing degree of the node
// 
// @return the number of ingoing edges
// 
- (unsigned long) ingoingDegree;

//
// Return the outgoing degree of the node
// 
// @return the number of output edges
// 
- (unsigned long) outgoingDegree;

//
// Return the degree of the node
// 
// @return the number of ingoing and outgoing edges
// 
- (unsigned long) degree;

//
// Add an ingoing edge
// 
// @param edge    the edge
// 
// @return the node object
// 
- (DGraphNode *) addIngoingEdge :(DGraphEdge *) edge;

//
// Remove an ingoing edge
// 
// @param edge    the edge
// 
// @return success
// 
- (BOOL) removeIngoingEdge :(DGraphEdge *) edge;

//
// Add an outgoing edge
// 
// @param edge    the edge
// 
// @return the node object
// 
- (DGraphNode *) addOutgoingEdge :(DGraphEdge *) edge;

//
// Remove an outgoing edge
// 
// @param edge    the edge
// 
// @return success
// 
- (BOOL) removeOutgoingEdge :(DGraphEdge *) edge;


//// Edge iterators

//
// Return the ingoing edge iterator
// 
// @return a (new) iterator on the ingoing edges
//
- (DListIterator *) ingoingEdges;

//
// Return the outgoing edge iterator
// 
// @return a (new) list iterator on the outgoing edges
// 
- (DListIterator *) outgoingEdges;

@end

//
// The graph edge object stores an object in an edge of a graph.
//

@interface DGraphEdge : Object
{
@private
  DGraphNode    *_source;     // the source node
  DGraphNode    *_target;     // the target node
  DText         *_name;       // the name (set by DGraph)
  DText         *_label;      // the (graphviz) label 
  double         _weight;     // the weight of the edge (> 0.0)
  DText         *_attributes; // the (graphviz) attributes
  id             _object;     // the stored object
}

//// Constructors

//
// Initialise an empty graph edge
//
// @return the object
//
- (DGraphEdge *) init;

//
// Initialise a graph edge with a label, weight and object
//
// @param label    the label
// @param weight   the weight (> 0.0)
// @param object   the object
//
// @return the object
//
- (DGraphEdge *) init :(const char *) label :(double) weight :(id) object;

//
// Initialise a graph edge with a label, weight, attributes and object
//
// @param label        the label 
// @param attributes   the (graphviz) attributes
// @param weight       the weight (> 0.0)
// @param object       the object
//
// @return the object
//
- (DGraphEdge *) init :(const char *) label :(const char *) attributes :(double) weight :(id) object;


//// Copy related methods

//
// Deepen a copy of the graph edge (not implemented)
//
// @return the object
//
- deepen;


//// Destructor

//
// Free the graph edge (the stored object is NOT freed)
//
// @return the object
//
- shallowFree;

//
// Free the graph edge (the stored object is freed)
//
// @return the object
//
- free;


//// Member methods

//
// Return the name of the edge (used by DGraph)
// 
// @return the name of the node
// 
- (const char *) name;

//
// Set the name of the edge (used by DGraph)
// 
// @param name    the name
// 
// @return the edge object
// 
- (DGraphEdge *) name :(const char *) name;

//
// Return the (graphviz) label of the graph edge
//
// @return the label (or NULL)
//
- (const char *) label;

//
// Set the (graphviz) label of the graph edge
//
// @param label   the label
// 
// Return the edge object
// 
- (DGraphEdge *) label :(const char *) label;

//
// Return the weight of the graph edge
//
// @return the weight
//
- (double) weight;

//
// Set the weight of the graph edge
//
// @param weight  the weight (> 0.0)
// 
// Return the edge object
// 
- (DGraphEdge *) weight :(double) weight;

//
// Return the (graphviz) attributes of the edge
// 
// @return the attributes (or NULL)
// 
- (const char *) attributes ;

//
// Set the (graphviz) attributes for the edge
// 
// @param attributes    the attributes
// 
// @return the edge object
// 
- (DGraphEdge *) attributes :(const char *) attributes;

//
// Return the stored object in the edge
// 
// @return the (stored) object (or nil)
// 
- (id) object;

//
// Set the object for the edge
// 
// @param object    the object
// 
// @return the previous stored object
// 
- (id) object :(id) object;


//// main methods

//
// Connect the source and target nodes by the edge
// 
// @param source    the source node (!= nil)
// @param target    the target node (!= nil)
// 
// @return success
// 
- (BOOL) connect :(DGraphNode *) source :(DGraphNode *) target;

//
// Disconnect the edge from the source and target node
// 
// @return the edge object
// 
- (DGraphEdge *) disconnect ;

//
// Return the source node
// 
// @return the source node
// 
- (DGraphNode *) source;

//
// Return the target node
// 
// @return the target node (or nil)
// 
- (DGraphNode *) target;

//
// Reverse the edge
// 
// @return the edge object
// 
- (DGraphEdge *) reverse;

@end

  
#include "ofc/DTextWritable.h"
  
//
// The graph collection stores objects in directed graph with nodes and edges. 
// Both the nodes as the edges can store objects. The graph collection can also 
// be used as graph data structure. Using the toDot method, the graph structure can 
// be exported to a dot-file which can be visualized with the software of 
// @link(www.graphviz.org). IMPORTANT: the node and edge objects are owned by the graph 
// object. Do NOT free them outside the scope of the graph object.
// 

@interface DGraph : Object
{
@private
  DList         *_nodes;       // the list with the graph nodes
  DList         *_edges;       // the list with the edges
  
  DText         *_attributes;  // the optional (graphviz) attributes
  
  long           _nodeCounter; // the node counter (for the node name)
  long           _edgeCounter; // the edge counter (for the edge name)
}

//// Constructors

//
// Initialise an empty graph 
//
// @return the object
//
- (DGraph *) init;

//
// Initialise a graph with attributes 
//
// @param attributes   the (graphviz) attributes
//
// @return the object
//
- (DGraph *) init :(const char *) attributes;


//// Copy related methods

//
// Deepen a copy of the graph  (not implemented)
//
// @return the object
//
- deepen;


//// Destructor

//
// Free the graph (the stored objects are NOT freed)
//
// @return the object
//
- shallowFree;

//
// Free the graph  (the stored objects are freed)
//
// @return the object
//
- free;


//// Member methods

//
// Return the (graphviz) attributes of the graph
// 
// @return the attributes (or NULL)
// 
- (const char *) attributes ;

//
// Set the (graphviz) attributes for the graph
// 
// @param attributes    the attributes (or NULL)
// 
// @return the graph object
// 
- (DGraph *) attributes :(const char *) attributes;


//// Main methods

//
// Add a (new) node to the graph (graph is the owner of the node)
// 
// @param label       the label for the node (or NULL)
// @param attributes  the attributes for the node (or NULL)
// @param object      the object for the node (or nil)
// 
// @return the (new) node object
// 
- (DGraphNode *) addNode :(const char *) label :(const char *) attributes :(id) object;

//
// Add a (new) node to the graph (graph is the owner of the node)
// 
// @param node        the graph node
// 
// @return success
// 
- (BOOL) addNode :(DGraphNode *) node;

//
// Add a (new) edge to the graph (graph is the owner of the edge object)
// 
// @param label       the label for the edge (or NULL)
// @param attributes  the attributes for the edge (or NULL)
// @param weight      the weight for the edge
// @param object      the object for the edge (or nil)
// @param source      the source node
// @param target      the target node
// 
// @return the (new) edge object (or nil)
// 
- (DGraphEdge *) addEdge :(const char *) label :(const char *) attributes :(double) weight :(id) object :(DGraphNode *) source :(DGraphNode *) target;

//
// Add a (new) edge to the graph (graph is the owner of the edge object)
// 
// @param edge        the graph edge
// @param source      the source node 
// @param target      the target node
// 
// @return success
// 
- (BOOL) addEdge :(DGraphEdge *) edge :(DGraphNode *) source :(DGraphNode *) target;

//
// Reroute an existing edge in the graph
//  
// @param edge        the edge
// @param source      the new source for the edge
// @param target      the new target for the edge
// 
// @return success
// 
- (BOOL) reroute :(DGraphEdge *) edge :(DGraphNode *) source :(DGraphNode *) target;

//
// Determine the shortest path between two nodes; a (new) list with REFERENCES
// to the nodes in the shortest path are returned; IMPORTANT: free this list 
// with shallowFree
// 
// @param from      the start node
// @param to        the end node
// @param weight    the destination for the path weight (or NULL)
// 
// @return a (new) list with REFERENCES to the nodes in the shortes path (or nil)
//
- (DList *) shortestPath :(double *) weight :(DGraphNode *) from :(DGraphNode *) to;

//
// Check if the graph has a node
// 
// @param node      the graph node
// 
// @return has it ?
// 
- (BOOL) hasNode :(DGraphNode *) node;

//
// Check if the graph has an edge
// 
// @param edge      the graph edge
// 
// @return has it ?
// 
- (BOOL) hasEdge :(DGraphEdge *) edge;

//
// Remove an node from the graph (node must be unconnected)
// 
// @param node      the graph node
// 
// @return the object stored in the node (or nil)
// 
- (id) removeNode :(DGraphNode *) node;

//
// Remove an edge from the graph (auto disconnect)
// 
// @param edge      the edge node
// 
// @return the object stored in the edge (or nil)
// 
- (id) removeEdge :(DGraphEdge *) edge;


//// Iterators

//
// Return a (new) iterator on all the nodes in the graph
// 
// @return a (new) iterator
// 
- (DListIterator *) nodes;

//
// Return a (new) iterator on all the edges in the graph
// 
// @return a (new) list iterator
// 
- (DListIterator *) edges;


//// Export methods

//
// Export to DOT 
// 
// @param writer    the output object
// 
// @return success ?
// 
- (BOOL) toDot :(id <DTextWritable>) writer;

@end
#endif
