#ifndef _DSHA1_H
#define _DSHA1_H

// This file was automatically generated by objch.

//==============================================================================
//
//      DSHA1 - the NIST's Secure Hash Algorithm class in the ofc-library
//
//               Copyright (C) 2002  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2005/11/12 10:04:04 $ $Revision: 1.5 $
//==============================================================================

#include "ofc/config.h"

#include "ofc/DText.h"
#include "ofc/DData.h"

//
// The DSHA1 class implements a number of methods for using the NIST's SHA
// algorithm. The code is based on 'SHA-1 in C' by Steve Reid (steve[at]edmweb.com)
// which is in Public Domain.
//

@interface DSHA1 : Object
{
@private
  unsigned long _state[5];
  unsigned long _count[2];
  unsigned char _buffer[64];
}


//// Constructors

//
// Initialise the sha object
//
// @return the object
//
- (DSHA1 *) init;

//
// Initialise the sha object with a string
//
// @param cstring  the c-string
//
// @return the object
//
- (DSHA1 *) init :(const char *) cstring;

//
// Initialise the sha object with data
//
// @param data     the data ('\0' allowed)
// @param length   the length of the data
//
// @return the object
//
- (DSHA1 *) init :(const unsigned char *) data :(unsigned long) length;


//// Update methods

//
// Update the object with data. Repeated calls can be compared
// to a single call in which the strings are concatted
//
// @param data     the data to be feeded to sha
// @param length   the length of the data
//
// @return the object
//
- (DSHA1 *) update :(const unsigned char *) data :(unsigned long) length;

//
// Update the object with a c-string. Repeated calls can be compared
// to a single call in which the strings are concatted
//
// @param cstring  the c-string to be feeded to sha1
//
// @return the object
//
- (DSHA1 *) update :(const char *) cstring;


//// Digest methods

//
// Return the (current) digest of all data passed to the object. 
//
// @return the (new) data string object with the digest
//
- (DData *) digest;

//
// Returns the (current) digest of all data passed to the object. The
// returning string contains the digest in hexadecimal ascii characters.
//
// @return the (new) string object with the digest
//
- (DText *) hexdigest;

@end
#endif
