#ifndef _DSCORE_H
#define _DSCORE_H

// This file was automatically generated by objch.

//==============================================================================
//
//          DScore - the score datatype in the ofc-library
//
//               Copyright (C) 2005  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2005/05/05 07:12:19 $ $Revision: 1.2 $
//
//==============================================================================

#include "ofc/config.h"


//
// The DScore class implements a score in a (discrete) distribution. The score
// object is updated with new values and calculates the length, the average or 
// mean, the variance, the standard deviation, the sum and the squared sum. A 
// score object is not a collection: the update values are not stored in the
// object; the state of the object is updated with the new value.
// 

@interface DScore : Object
{
@private
  long                _length;     // the number of values in the object
  double              _min;        // the minimum value of the range
  double              _max;        // the maximum value of the range
  double              _sum;        // the sum of the values
  double              _sumSquared; // the sum of the squared values
  double              _percentage; // the percentage of this score in the distribution (0.0 .. 100.0)
}


//// Constructors

//
// Initialise the object with no range
//
// @return the object
//
- (DScore *) init;

//
// Initialise with a range
//
// @param min      the minimal value of the range
// @param max      the maximum value of the range
// 
// @return the object
//
- (DScore *) init :(double) min :(double) max;

//
// Initialise with a discrete value
//
// @param value    the discrete value
// 
// @return the object
//
- (DScore *) init :(int) value;


//// Member methods

//
// Return the mininal value
// 
// @return the minimal value
//
- (double) min;

//
// Return the maximum value
// 
// @return the maximum value
// 
- (double) max;

//
// Return the number of the values
// 
// @return the number
// 
- (int) length;

//
// Return the sum of the values
// 
// @return the sum
//
- (double) sum;

//
// Return the sum of the squared values
// 
// @return the sum of the squared values
// 
- (double) sumSquared;

//
// Return the percentage of this score in the full distribution
// 
// @return the percentage (0.0 .. 100.0)
// 
- (double) percentage;


//// Set methods

//
// Set the range (only when the length is zero)
//
// @param min      the minimal value
// @param max      the maximal value
//
// @return success
//
- (BOOL) range :(double) min :(double) max;

//
// Set the range with a discrete value (only when the length is zero)
//
// @param value    the discrete value
//
// @return success
//
- (BOOL) range :(int) value;

//
// Reset the length and sums to zero
// 
// @return the object
// 
- (DScore *) reset;

//
// Set the number of the values in the distribution
// 
// @param length   the number of values in the distribution
// 
// @return the object
// 
- (DScore *) distribution :(long) length;

//
// Update the object with a value
// 
// @param value    the update value
// 
// @return success
// 
- (BOOL) update :(double) value;


//// Calculation methods

//
// Calculate the mean of the values
// 
// @return the mean value
// 
- (double) mean;

//
// Calculate the average of the values
// 
// @return the average value
// 
- (double) average;

//
// Calculate the variance of the values
// 
// @return the variance
// 
- (double) variance;

//
// Calculate the standard deviation of the values
// 
// @return the standard deviation
// 
- (double) standardDeviation;

@end
#endif
