#ifndef _DTABLE_H
#define _DTABLE_H

// This file was automatically generated by objch.

//==============================================================================
//
//         DTable - the 2d - array collection in the ofc-library
//
//               Copyright (C) 2003  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2005/02/05 06:25:38 $ $Revision: 1.5 $
//
//==============================================================================

#include "ofc/config.h"

// The table collection stores objects in an 2d array that can grow. 
// Objects can be accessed by an index. Due to the easy access there 
// is no iterator.

@interface DTable : Object
{
@private
  id            *_objects;  // the stored objects
  int            _columns;  // the number of columns
  int            _rows;     // the number of rows
  long           _length;   // the length of the table
}


//// Constructors

//
// Initialise an empty table
//
// @return the object
//
- (DTable *) init;

//
// Initialise a table with columns and rows
//
// @param columns  the number of columns
// @param rows     the number of rows
//
// @return the object
//
- (DTable *) init :(int) columns :(int) rows;


//// Copy related methods

//
// Deepen a copy of the table
//
// @return the object
//
- deepen;


//// Destructor

//
// Free the table, not the stored objects
//
// @return the object
//
- shallowFree;

//
// (Deep) free the table (incl. the objects)
//
// @return the object
//
- free;


//// Member methods

//
// Return the length of the table (columns * rows)
//
// @return the length
//
- (long) length;

//
// Return the number of columns
// 
// @return the number of columns
// 
- (int) columns;

//
// Set the number of columns
// 
// @param columns  the number of columns
// 
// @return the object
// 
- (DTable *) columns :(int) columns;

//
// Return the number of rows
// 
// @return the number of rows
// 
- (int) rows;

//
// Set the number of rows
// 
// @param rows     the number of rows
// 
// @return the object
// 
- (DTable *) rows :(int) rows;

//
// Check if an index is valid
//
// @param col      the column (0..)
// @param row      the row (0..)
//
// @return is it valid ?
//
- (BOOL) isValid :(int) col :(int) row;


//// Main methods

//
// Set an object in the table
//
// @param col      the column (0..)
// @param row      the row (0..)
// @param obj      the object to be set
//
// @return the previous stored object in the table (or nil)
//
- (id) set :(int) col :(int) row :(id) obj;

//
// Get an object from the table
//
// @param col      the column (0..)
// @param row      the row (0..)
//
// @return the object in the table (or nil)
//
- (id) get :(int) col :(int) row;

//
// Count the number of occurences of an object
//
// @param obj      the object to be counted
//
// @return the number of occurences
//
- (long) count :(id) obj;

//
// Check if an object is in the table
//
// @param obj      the object to be checked
//
// @return is it in the table ?
//
- (BOOL) has :(id) obj;

//
// Perform a selector on each object in the table (row after row)
//
// @param sel      the selector
//
// @return the object
//
- (DTable *) each :(SEL) sel;

@end
#endif
