#ifndef _DXML_H
#define _DXML_H

// This file was automatically generated by objch.

//==============================================================================
//
//            DXML - the XML reader, writer and tree in the ofc-library
//
//               Copyright (C) 2004  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2005/11/14 19:55:41 $ $Revision: 1.29 $
//
//==============================================================================

#include "ofc/config.h"

#include "ofc/DList.h"

#include "ofc/DTextReadable.h"
#include "ofc/DTextWritable.h"

#if defined(HAVE_EXPAT_H) && defined(HAVE_LIBEXPAT)
# define HAVE_DXML (1)
#endif


#ifdef HAVE_DXML

#include "expat.h"

#define DXML_ELEMENT      (1)
#define DXML_ATTRIBUTE    (2)
#define DXML_TEXT         (3)
#define DXML_CDATA        (4)
#define DXML_ENTITY_REF   (5) // Not used
#define DXML_ENTITY       (6) // Not used
#define DXML_PI           (7)
#define DXML_COMMENT      (8)
#define DXML_DOCUMENT     (9)
#define DXML_DOC_TYPE     (10) // Not used
#define DXML_DOC_FRAGMENT (11) // Not used
#define DXML_NOTATION     (12) // Not used
#define DXML_NAMESPACE    (13) // Not in DOM

typedef XML_Char   DXMLChar;

//
// The DXMLHandler protocol specifies the methods a xml handler
// must implement for processing the contents of a xml source.
//

@protocol DXMLHandler

//
// Process the start of a document
// 
// @param version     the version number
// @param encoding    the encoding
// @param standalone  is the xml document standalone ? (-1 = missing)
// 
// @return success
// 
- (BOOL) startDocument :(const DXMLChar *) version :(const DXMLChar *) encoding :(int) standalone;

//
// Process the end of a document
// 
// @return success
// 
- (BOOL) endDocument;

//
// Process the start of an element
// 
// @param name     the name of the element
// 
// @return success
// 
- (BOOL) startElement :(const DXMLChar *) name;

//
// Process an attribute for an element (after a startElement)
// 
// @param attribute  the attribute
// @param value      the value for the attribute
// 
// @return success
// 
- (BOOL) attribute :(const DXMLChar *) attribute :(const DXMLChar *) value;

//
// Process the end of an element
// 
// @return success
//
- (BOOL) endElement;

//
// Process a string of characters
// 
// @param text     the text of the characters
// 
// @return success
// 
- (BOOL) characters :(const DXMLChar *) text;

//
// Process a comment
// 
// @param text     the text of the comment
// 
// @return success
// 
- (BOOL) comment :(const DXMLChar *) text;

//
// Process a processing instruction
// 
// @param target   the target
// @param value    the value
// 
// @return success
// 
- (BOOL) processingInstruction :(const DXMLChar *) target :(const DXMLChar *) value;

//
// Process the start of a CDATA section
// 
// @return success
// 
- (BOOL) startCDATA;

//
// Process the end of a CDATA section
// 
// @return success
// 
- (BOOL) endCDATA;

//
// Start of a namespace declaration
// 
// @param prefix   the prefix
// @param uri      the uri
// 
// @return success
// 
- (BOOL) startNamespace :(const DXMLChar *) prefix :(const DXMLChar *) uri;

//
// End of a namespace declaration
// 
// @return success
// 
- (BOOL) endNamespace;

//
// Process unparsed text in the xml document (e.g. the DTD)
// 
// @param text     the unparsed text
// 
// @return success
// 
- (BOOL) unparsed :(const DXMLChar *) text;

// 
// An error is found during the parsing of the file
// 
// @param number       the error number
// @param name         the name of the config file
// @param lineNumber   the line number in the config file
// @param columnNumber the column number in the config file
// 
// @return success
// 
- (void) error :(int) number :(const char *) name :(int) lineNumber :(int) columnNumber;

@end




//
// The DXMLReader class implements methods for parsing a xml file. The class 
// uses expat for the low level parsing. The parsing of DTDs and External 
// Entities are not supported. The method 'characters' returns the exact 
// contents between the start elements and end element, including linefeed 
// and indent spacing. The method 'unparsed' returns everything that could 
// not be parsed, including DTDs and External Entities References. Use a character 
// that can't be part of an URI e.g. '|' for the namespace separator.
//

@interface DXMLReader : Object
{
@private
  id <DXMLHandler>     _handler;     // the reference to the xml handler
  int                  _bufferSize;  // the size of the buffer (def. 8192)
  XML_Parser           _parser;      // the xml parser
  DText               *_name;        // the name of the source
  DXMLChar             _separator;   // the namespace separator (def. \0 = no namespace)
  DXMLChar            *_encoding;    // the encoding to be used (def. NULL = from xml file)
  DText               *_characters;  // the characters buffer
  DText               *_unparsed;    // the unparsed buffer
}
#endif

#ifdef HAVE_DXML

//// Constructors

//
// Initialise a xml reader
//
// @return the object
//
- (DXMLReader *) init;


//// Copy constructor

//
// Deepen a copy of the object (not implemented)
//
// @return the object
//
- deepen;


//// Deconstructor

//
// Free the object
//
// @return the object
//
- free;


//// Member methods

//
// Set the buffer size for the parser
// 
// @param size     the buffer size
// 
// @return the object
// 
- (DXMLReader *) bufferSize :(int) size;

//
// Get the buffer size for the parser
// 
// @return the size
// 
- (int) bufferSize;

//
// Set the encoding overrider 
// 
// @param encoding   the encoding to be used
// 
// @return the object
// 
- (DXMLReader *) encoding :(const DXMLChar *) encoding;

//
// Get the encoding used for the xml documents
// 
// @return the encoding overrider (or NULL)
// 
- (const DXMLChar *) encoding;


//// Parser methods

//
// Parse a XML file
// 
// @param source     the source of the xml file
// @param name       the name of the xml file
// @param handler    the xml handler
// @param separator  the namespace separator
// 
// @return success
// 
- (BOOL) parse :(id <DTextReadable>) source :(const char *) name :(id <DXMLHandler>) handler :(DXMLChar) separator;

//
// Return the current line number during parsing
// 
// @return the current line number (or 0)
// 
- (int) lineNumber;

//
// Return the current column number during parsing
// 
// @return the current column number
// 
- (int) columnNumber;

//
// Return the name of the source during parsing
// 
// @return the current source name
// 
- (const char *) name;

//
// Translate the error to a text string
// 
// @param error    the error number
// 
// @return the text string
// 
+ (const char *) errorToString :(int) error;

@end
#endif

#ifdef HAVE_DXML

//
// The DXMLNode class implements methods for storing information
// from a XML source in a node. The following scheme is used by
// the class.
// <pre>
//        Type              Name           Value      Remarks
//        ----------------------------------------------------
//        DXML_ELEMENT      Name           NULL
//        DXML_ATTRIBUTE    Name           value
//        DXML_TEXT         #text          text       1)
//        DXML_CDATA        #cdata         text       1)
//        DXML_ENTITY_REF                             Not used
//        DXML_ENTITY                                 Not used
//        DXML_PI           target         value
//        DXML_COMMENT      #comment       text       1)
//        DXML_DOCUMENT     #document      value      1)
//        DXML_DOC_TYPE                               Not used
//        DXML_DOC_FRAGMENT                           Not used
//        DXML_NOTATION                               Not used
//        DXML_NAMESPACE    prefix         uri        2)
//        ----------------------------------------------------
//        1) Name parameter for set::: method is ignored
//        2) Not present in DOM
// </pre>

@interface DXMLNode : Object
{
  int     _type;   // the type of the node DXML_ELEMENT .. DXML_NAMESPACE
  DText  *_name;   // the 'name'of the node 
  DText  *_value;  // the value of the node
}
#endif
#ifdef HAVE_DXML

//// Constructors

//
// Initialise an empty xml node
//
// @return the object
//
- (DXMLNode *) init;

//
// Initialise a xml node with a type, a name and a value
// 
// @param type       the xml node type (DXML_..)
// @param name       the name of the node
// @param value      the value of the node (or NULL)
//
// @return the object
//
- (DXMLNode *) init :(int) type :(const char *) name :(const char *) value;


//// Copy constructor

//
// Deepen a copy of the object
//
// @return the object
//
- deepen;


//// Deconstructor

//
// Free the object
//
// @return the object
//
- free;


//// Member methods

//
// Return the type of the xml node
// 
// @return the type (DXML_ELEMENT..DXML_NOTATION)
// 
- (int) type;

//
// Return the name of the xml node
// 
// @return the name of the node (or NULL)
// 
- (const char *) name;

//
// Return the value of the xml node
// 
// @return the value of the node (or NULL)
// 
- (const char *) value;


//// Main method

//
// Set the xml node (see table for parameters)
// 
// @param type       the xml node type (DXML_..)
// @param name       the name of the node (or NULL)
// @param value      the value of the node (or NULL)
//
// @return the object
//
- (DXMLNode *) set :(int) type :(const char *) name :(const char *) value;

@end
#endif

#ifdef HAVE_DXML

//
// The xml writer class implements methods for writing a xml file. This class 
// does NOT check the correctness of the written xml. The method 'characters' 
// translates the special characters (<,>,&,",'), the method 'unparsed' does 
// not. With the last method an entity or character reference can be written.
// Linefeeds are never added by the writer. Linefeeds must be supplied by the 
// calling software by using 'characters:' or 'unparsed:'. Use a character 
// that can't be part of an URI e.g. '|' for the namespace separator.
//
   
@interface DXMLWriter : Object <DXMLHandler>
{
@private
  id <DTextWritable>  _dest;         // the config file
  DList              *_elements;     // the stack with the element names
  DList              *_namespaces;   // the stack with the namespaces
  DList              *_unprocessed;  // the list with the unprocessed namespaces
  BOOL                _closeElement; // should the last element still be closed ?
  BOOL                _inCData;      // is the writer inside a CDATA section ?
  char                _separator;    // the namespace separator (or EOS for none)
}
#endif
#ifdef HAVE_DXML

//// Constructors

//
// Initialise a xml writer
//
// @return the object
//
- (DXMLWriter *) init;

//
// Initialise a xml writer with a destination file
// 
// @param destination  the xml file
// @param separator    the namespace separator
//
// @return the object
//
- (DXMLWriter *) init :(id <DTextWritable>) destination :(char) separator ;


//// Copy related methods

//
// Deepen a copied object (not implemented)
//
// @return the object
//
- deepen;


//// Deconstructor

//
// Free the object
//
// @return the object
//
- free;


//// Writer methods

//
// Start the writing of a xml file
// 
// @param destination  the xml file
// @param separator    the namespace separator (or EOS for no namespaces)
// 
// @return the object
// 
- (BOOL) start :(id <DTextWritable>) destination :(char) separator;

//
// Write the start of a document
// 
// @param version      the version number (or NULL)
// @param encoding     the encoding (or NULL)
// @param standalone   is the xml document standalone (-1 = missing) ?
// 
// @return success
// 
- (BOOL) startDocument :(const DXMLChar *) version :(const DXMLChar *) encoding :(int) standalone;

//
// Write the end of a document
// 
// @return success
// 
- (BOOL) endDocument;

//
// Write the start of an element
// 
// @param name     the name of the element
// 
// @return success
// 
- (BOOL) startElement :(const DXMLChar *) name;

//
// Write an attribute for an element
// 
// @param attribute  the attribute
// @param value      the value for the attribute
// 
// @return success
// 
- (BOOL) attribute :(const DXMLChar *) attribute :(const DXMLChar *) value;

//
// Write the end of an element
// 
// @return success
//
- (BOOL) endElement;

//
// Write a string of characters
// 
// @param text     the text of the characters
// 
// @return success
// 
- (BOOL) characters :(const DXMLChar *) text;

//
// Write a comment
// 
// @param text     the text of the comment
// 
// @return success
// 
- (BOOL) comment :(const DXMLChar *) text;

//
// Write a processing instruction
// 
// @param target   the target
// @param value    the value
// 
// @return success
// 
- (BOOL) processingInstruction :(const DXMLChar *) target :(const DXMLChar *) value;

//
// Write the start of a CDATA section
// 
// @return success
// 
- (BOOL) startCDATA;

//
// Write the end of a CDATA section
// 
// @return success
// 
- (BOOL) endCDATA;

//
// Start of a namespace declaration
// 
// @param prefix   the prefix
// @param uri      the uri
// 
// @return success
// 
- (BOOL) startNamespace :(const DXMLChar *) prefix :(const DXMLChar *) uri;

//
// End of a namespace declaration
// 
// @return success
// 
- (BOOL) endNamespace;

//
// Write unparsed text in the xml document (e.g. the DTD)
// Note: there is no translation whatsoever done on this text.
// 
// @param text     the unparsed text
// 
// @return success
// 
- (BOOL) unparsed :(const DXMLChar *) text;

// 
// An error is found during the parsing of the file
// 
// @param number       the error number
// @param name         the name of the config file
// @param lineNumber   the line number in the config file
// @param columnNumber the column number in the config file
// 
// @return success
// 
- (void) error :(int) number :(const char *) name :(int) lineNumber :(int) columnNumber;;

@end
#endif

#ifdef HAVE_DXML

#include "ofc/DTree.h"

//
// The XML Tree class implements methods for reading, storing, changing,
// removing and writing xml information. Due to the fact that every xml
// piece of information is stored in a xml node, large xml files can take
// a lot of memory. Unparsed information is not stored in the tree. Use 
// a character that can't be part of an URI e.g. '|' for the namespace 
// separator.
//

@interface DXMLTree : DTree <DXMLHandler>
{
@private
  DTreeIterator *_iter;      // the iterator in the xml tree
  BOOL           _inCData;   // CDATA section entered ?
  char           _separator; // the namespace separator (or EOS for no namespaces)
}

#endif
#ifdef HAVE_DXML

//// Constructors

//
// Initialise a config tree
//
// @return the object
//
- (DXMLTree *) init;

//
// Initialise a xml tree with a xml source
// 
// @param source     the xml source
// @param filename   the name of the xml source
// @param separator  the namespace separator (or EOS for no namespaces)
//
// @return the object
//
- (DXMLTree *) init :(id <DTextReadable>) source :(const char *) name :(char) separator;


//// Copy constructor

//
// Deepen a copy of the object
//
// @return the object
//
- deepen;


//// Deconstructor

//
// Free the object (including the stored objects)
//
// @return the object
//
- free;

//
// Free the object (without the stored objects)
//
// @return the object
//
- shallowFree;


//// Xml tree reading and writing

//
// Read the xml from a source
//  
// @param source   the source of the xml
// @param name     the name of the source
// @param separtor the namespace separator (or EOS for no namespaces)
// 
// @return success
// 
- (BOOL) read :(id <DTextReadable>) source :(const char *) name :(char) separator;

//
// Write the xml to a destination
// 
// @param destination the destination of the xml
// @param name        the name of the destination (not used)
// 
// @return success
// 
- (BOOL) write :(id <DTextWritable>) destination :(const char *) name;


//// Call back methods for the xml parser (private)

//
// Process the start of a document
// 
// @param version     the version number
// @param encoding    the encoding
// @param standalone  is the xml document standalone ? (-1 = missing)
// 
// @return success
// 
- (BOOL) startDocument :(const DXMLChar *) version :(const DXMLChar *) encoding :(int) standalone;

//
// Process the end of a document
// 
// @return success
// 
- (BOOL) endDocument;

//
// Process the start of an element
// 
// @param name     the name of the element
// 
// @return success
// 
- (BOOL) startElement :(const DXMLChar *) name;

//
// Process an attribute for an element (after a startElement)
// 
// @param attribute  the attribute
// @param value      the value for the attribute
// 
// @return success
// 
- (BOOL) attribute :(const DXMLChar *) attribute :(const DXMLChar *) value;

//
// Process the end of an element
// 
// @return success
//
- (BOOL) endElement;

//
// Process a string of characters
// 
// @param text     the text of the characters
// 
// @return success
// 
- (BOOL) characters :(const DXMLChar *) text;

//
// Process a comment
// 
// @param text     the text of the comment
// 
// @return success
// 
- (BOOL) comment :(const DXMLChar *) text;

//
// Process a processing instruction
// 
// @param target   the target
// @param value    the value
// 
// @return success
// 
- (BOOL) processingInstruction :(const DXMLChar *) target :(const DXMLChar *) value;

//
// Process the start of a CDATA section
// 
// @return success
// 
- (BOOL) startCDATA;

//
// Process the end of a CDATA section
// 
// @return success
// 
- (BOOL) endCDATA;

//
// Start of a namespace declaration
// 
// @param prefix   the prefix
// @param uri      the uri
// 
// @return success
// 
- (BOOL) startNamespace :(const DXMLChar *) prefix :(const DXMLChar *) uri;;

//
// End of a namespace declaration
// 
// @return success
// 
- (BOOL) endNamespace;

//
// Process unparsed text in the xml document (e.g. the DTD)
// 
// @param text     the unparsed text
// 
// @return success
// 
- (BOOL) unparsed :(const DXMLChar *) text;

//
// An error is found during the parsing of the file
// 
// @param number       the error number
// @param name         the name of the config file
// @param lineNumber   the line number in the config file
// @param columnNumber the column number in the config file
// 
// @return success
// 
- (void) error :(int) number :(const char *) name :(int) lineNumber :(int) columnNumber;;

@end
#endif
#endif
